@@ -4,8 +4,11 @@ namespace Icinga\Module\Director\Clicommands;
 
 use Icinga\Application\Benchmark;
 use Icinga\Module\Director\Cli\Command;
+use Icinga\Module\Director\Core\Json;
+use Icinga\Module\Director\Deployment\DeploymentStatus;
 use Icinga\Module\Director\IcingaConfig\IcingaConfig;
-use Icinga\Module\Director\Util;
+use Icinga\Module\Director\Import\SyncUtils;
+use Icinga\Module\Director\Objects\DirectorDeploymentLog;
 
 /**
  * Generate, show and deploy Icinga 2 configuration
@@ -84,8 +87,16 @@ class ConfigCommand extends Command
     /**
      * Deploy the current configuration
      *
-     * Does nothing if config didn't change unless you provide
-     * the --force parameter
+     * USAGE
+     *
+     * icingacli director config deploy [--checksum <checksum>] [--force] [--wait <seconds>]
+     *
+     * OPTIONS
+     *
+     *   --checksum <checksum>  Optionally deploy a specific configuration
+     *   --force                Force a deployment, even when the configuration hasn't
+     *                          changed
+     *   --wait <seconds>       Optionally wait until Icinga completed it's restart
      */
     public function deployAction()
     {
@@ -94,7 +105,7 @@ class ConfigCommand extends Command
 
         $checksum = $this->params->get('checksum');
         if ($checksum) {
-            $config = IcingaConfig::load(Util::hex2binary($checksum), $db);
+            $config = IcingaConfig::load(hex2bin($checksum), $db);
         } else {
             $config = IcingaConfig::generate($db);
             $checksum = $config->getHexChecksum();
@@ -112,20 +123,60 @@ class ConfigCommand extends Command
             }
         }
 
-        if ($api->dumpConfig($config, $db)) {
-            printf("Config '%s' has been deployed\n", $checksum);
+        $deploymentLog = $api->dumpConfig($config, $db);
+        if (! $deploymentLog) {
+            $this->fail("Failed to deploy config '%s'", $checksum);
+        }
+        if ($timeout = $this->params->get('wait')) {
+            if (! ctype_digit($timeout)) {
+                $this->fail("--wait must be the number of seconds to wait'");
+            }
+            $deployed = $this->waitForStartupAfterDeploy($deploymentLog, $timeout);
+            if ($deployed !== true) {
+                $this->fail("Failed to deploy config '%s': %s\n", $checksum, $deployed);
+            }
+        }
+        printf("Config '%s' has been deployed\n", $checksum);
+    }
+
+    /**
+     * Checks the deployments status
+     */
+    public function deploymentstatusAction()
+    {
+        $db = $this->db();
+        $api = $this->api();
+        $status = new DeploymentStatus($db, $api);
+        $result = $status->getDeploymentStatus($this->params->get('configs'), $this->params->get('activities'));
+        if ($key = $this->params->get('key')) {
+            $result = SyncUtils::getSpecificValue($result, $key);
+        }
+
+        if (is_string($result)) {
+            echo "$result\n";
         } else {
-            $this->fail(
-                sprintf("Failed to deploy config '%s'\n", $checksum)
-            );
+            echo Json::encode($result, JSON_PRETTY_PRINT) . "\n";
         }
     }
 
-    public function filesAction()
-    {
-    }
-
-    public function fileAction()
+    private function waitForStartupAfterDeploy($deploymentLog, $timeout)
     {
+        $startTime = time();
+        while ((time() - $startTime) <= $timeout) {
+            $deploymentFromDB = DirectorDeploymentLog::load($deploymentLog->getId(), $this->db());
+            $stageCollected = $deploymentFromDB->get('stage_collected');
+            if ($stageCollected === null) {
+                usleep(500000);
+                continue;
+            }
+            if ($stageCollected === 'n') {
+                return 'stage has not been collected';
+            }
+            if ($deploymentFromDB->get('startup_succeeded') === 'y') {
+                return true;
+            }
+            return 'deployment failed during startup';
+        }
+        return 'deployment timed out';
     }
 }
