@@ -2,11 +2,13 @@
 
 namespace Icinga\Module\Director\Controllers;
 
+use Icinga\Data\Filter\Filter;
 use Icinga\Module\Director\Forms\IcingaServiceSetForm;
 use Icinga\Module\Director\Objects\IcingaHost;
 use Icinga\Module\Director\Objects\IcingaServiceSet;
 use Icinga\Module\Director\Web\Controller\ObjectController;
 use Icinga\Module\Director\Web\Form\DirectorObjectForm;
+use Icinga\Module\Director\Web\Table\IcingaHostsMatchingFilterTable;
 use Icinga\Module\Director\Web\Table\IcingaServiceSetHostTable;
 use Icinga\Module\Director\Web\Table\IcingaServiceSetServiceTable;
 use gipfl\IcingaWeb2\Link;
@@ -82,20 +84,31 @@ class ServicesetController extends ObjectController
             $set->getObjectName()
         );
 
-        IcingaServiceSetHostTable::load($set)->renderTo($this);
+        $table = IcingaServiceSetHostTable::load($set);
+        if ($table->count()) {
+            $table->renderTo($this);
+        }
+        $filter = $set->get('assign_filter');
+        if ($filter !== null && \strlen($filter) > 0) {
+            $this->content()->add(
+                IcingaHostsMatchingFilterTable::load(Filter::fromQueryString($filter), $this->db())
+            );
+        }
     }
 
     protected function addServiceSetTabs()
     {
+        if ($this->branch->isBranch()) {
+            return $this;
+        }
         $tabs = $this->tabs();
-        $name = $this->object->getObjectName();
         $tabs->add('services', [
             'url'       => 'director/serviceset/services',
-            'urlParams' => ['name' => $name],
+            'urlParams' => ['uuid' => $this->object->getUniqueId()],
             'label'     => 'Services'
         ])->add('hosts', [
             'url'       => 'director/serviceset/hosts',
-            'urlParams' => ['name' => $name],
+            'urlParams' => ['uuid' => $this->object->getUniqueId()],
             'label'     => 'Hosts'
         ]);
 
