@@ -12,6 +12,7 @@ use Icinga\Module\Director\Web\Form\DirectorObjectForm;
 use Icinga\Module\Director\Objects\IcingaHost;
 use Icinga\Module\Director\Objects\IcingaService;
 use Icinga\Module\Director\Objects\IcingaServiceSet;
+use Icinga\Module\Director\Web\Table\ObjectsTableHost;
 use ipl\Html\Html;
 use gipfl\IcingaWeb2\Link;
 use RuntimeException;
@@ -139,6 +140,14 @@ class IcingaServiceForm extends DirectorObjectForm
         } else {
             $this->addOverrideHint();
             $group = $this->getDisplayGroup('custom_fields');
+            if (! $group) {
+                foreach ($this->getDisplayGroups() as $groupName => $eventualGroup) {
+                    if (preg_match('/^custom_fields:/', $groupName)) {
+                        $group = $eventualGroup;
+                        break;
+                    }
+                }
+            }
             if ($group) {
                 $elements = $group->getElements();
                 $group->setElements([$this->getElement('inheritance_hint')]);
@@ -325,7 +334,7 @@ class IcingaServiceForm extends DirectorObjectForm
     protected function setupServiceElements()
     {
         if ($this->object) {
-            $objectType = $this->object->object_type;
+            $objectType = $this->object->get('object_type');
         } elseif ($this->preferredObjectType) {
             $objectType = $this->preferredObjectType;
         } else {
@@ -414,10 +423,6 @@ class IcingaServiceForm extends DirectorObjectForm
         $this->addHtmlHint($hint, ['name' => 'inheritance_hint']);
     }
 
-    /**
-     * @throws IcingaException
-     * @throws ProgrammingError
-     */
     protected function setupOnHostForSet()
     {
         $msg = $this->translate(
@@ -470,7 +475,7 @@ class IcingaServiceForm extends DirectorObjectForm
      */
     protected function setupHostRelatedElements()
     {
-        $this->addHidden('host_id', $this->host->id);
+        $this->addHidden('host', $this->host->getObjectName());
         $this->addHidden('object_type', 'object');
         $this->addImportsElement();
         $imports = $this->getSentOrObjectValue('imports');
@@ -493,13 +498,7 @@ class IcingaServiceForm extends DirectorObjectForm
              ->addExtraInfoElements()
              ->setButtons();
 
-        if ($this->hasBeenSent()) {
-            $name = $this->getSentOrObjectValue('object_name');
-            if (!strlen($name)) {
-                $this->setElementValue('object_name', end($imports));
-                $this->object->object_name = end($imports);
-            }
-        }
+        $this->setDefaultNameFromTemplate($imports);
     }
 
     /**
@@ -543,13 +542,7 @@ class IcingaServiceForm extends DirectorObjectForm
                 ->addExtraInfoElements();
         }
 
-        if ($this->hasBeenSent()) {
-            $name = $this->getSentOrObjectValue('object_name');
-            if (!strlen($name)) {
-                $this->setElementValue('object_name', end($imports));
-                $this->object->object_name = end($imports);
-            }
-        }
+        $this->setDefaultNameFromTemplate($imports);
     }
 
     public function setServiceSet(IcingaServiceSet $set)
@@ -586,14 +579,14 @@ class IcingaServiceForm extends DirectorObjectForm
     protected function addHostObjectElement()
     {
         if ($this->isObject()) {
-            $this->addElement('select', 'host_id', array(
+            $this->addElement('select', 'host', [
                 'label'       => $this->translate('Host'),
                 'required'    => true,
                 'multiOptions' => $this->optionalEnum($this->enumHostsAndTemplates()),
                 'description' => $this->translate(
                     'Choose the host this single service should be assigned to'
                 )
-            ));
+            ]);
         }
 
         return $this;
@@ -695,10 +688,36 @@ class IcingaServiceForm extends DirectorObjectForm
 
     protected function enumHostsAndTemplates()
     {
-        return array(
-            $this->translate('Templates') => $this->db->enumHostTemplates(),
-            $this->translate('Hosts')     => $this->db->enumHosts(),
-        );
+        if ($this->branch && $this->branch->isBranch()) {
+            return $this->enumHosts();
+        }
+
+        return [
+            $this->translate('Templates') => $this->enumHostTemplates(),
+            $this->translate('Hosts')     => $this->enumHosts(),
+        ];
+    }
+
+    protected function enumHostTemplates()
+    {
+        $names = array_values($this->db->enumHostTemplates());
+        return array_combine($names, $names);
+    }
+
+    protected function enumHosts()
+    {
+        $db = $this->db->getDbAdapter();
+        $table = new ObjectsTableHost($this->db);
+        $table->setAuth($this->getAuth());
+        if ($this->branch && $this->branch->isBranch()) {
+            $table->setBranchUuid($this->branch->getUuid());
+        }
+        $result = [];
+        foreach ($db->fetchAll($table->getQuery()) as $row) {
+            $result[$row->object_name] = $row->object_name;
+        }
+
+        return $result;
     }
 
     protected function enumServicegroups()
@@ -715,10 +734,6 @@ class IcingaServiceForm extends DirectorObjectForm
         return $db->fetchPairs($select);
     }
 
-    /**
-     * @throws IcingaException
-     * @throws ProgrammingError
-     */
     protected function succeedForOverrides()
     {
         $vars = array();
@@ -739,7 +754,7 @@ class IcingaServiceForm extends DirectorObjectForm
                 $this->translate($host->getObjectName())
             );
 
-            $host->store();
+            $this->getDbObjectStore()->store($host);
         } else {
             if ($this->isApiRequest()) {
                 $this->setHttpResponseCode(304);
@@ -751,16 +766,27 @@ class IcingaServiceForm extends DirectorObjectForm
         $this->redirectOnSuccess($msg);
     }
 
-    /**
-     * @throws IcingaException
-     * @throws ProgrammingError
-     */
     public function onSuccess()
     {
         if ($this->providesOverrides()) {
-            return $this->succeedForOverrides();
+            $this->succeedForOverrides();
+            return;
         }
 
-        return parent::onSuccess();
+        parent::onSuccess();
+    }
+
+    /**
+     * @param array $imports
+     */
+    protected function setDefaultNameFromTemplate($imports)
+    {
+        if ($this->hasBeenSent()) {
+            $name = $this->getSentOrObjectValue('object_name');
+            if (!strlen($name)) {
+                $this->setElementValue('object_name', end($imports));
+                $this->object->set('object_name', end($imports));
+            }
+        }
     }
 }
