@@ -25,6 +25,7 @@ class RestApiResponse
     {
         $response = new static;
         $response->errorMessage = $error;
+
         return $response;
     }
 
@@ -33,6 +34,17 @@ class RestApiResponse
         return $this->extractResult($this->results, $desiredKey, $filter);
     }
 
+    public function getRaw($key = null, $default = null)
+    {
+        if ($key === null) {
+            return $this->results;
+        } elseif (isset($this->results[0]) && property_exists($this->results[0], $key)) {
+            return $this->results[0]->$key;
+        } else {
+            return $default;
+        }
+    }
+
     public function getSingleResult()
     {
         if ($this->isErrorCode($this->results[0]->code)) {
@@ -86,7 +98,12 @@ class RestApiResponse
         $result = @json_decode($json);
         if ($result === null) {
             $this->setJsonError();
-            throw new IcingaException('Parsing JSON result failed: ' . $this->errorMessage);
+            // <h1>Bad Request</h1><p><pre>bad version</pre></p>
+            throw new IcingaException(
+                'Parsing JSON result failed: '
+                . $this->errorMessage
+                . ' (Got: ' . substr($json, 0, 60) . ')'
+            );
         }
         if (property_exists($result, 'error')) {
             if (property_exists($result, 'status')) {
