@@ -3,11 +3,14 @@
 namespace Icinga\Module\Director\Data\Db;
 
 use Icinga\Exception\NotFoundError;
+use Icinga\Module\Director\Data\InvalidDataException;
 use Icinga\Module\Director\Db;
+use Icinga\Module\Director\Db\Branch\UuidLookup;
 use Icinga\Module\Director\Exception\DuplicateKeyException;
-use Icinga\Module\Director\Util;
 use InvalidArgumentException;
 use LogicException;
+use Ramsey\Uuid\Uuid;
+use Ramsey\Uuid\UuidInterface;
 use RuntimeException;
 use Zend_Db_Adapter_Abstract;
 use Zend_Db_Exception;
@@ -69,6 +72,9 @@ abstract class DbObject
      */
     protected $autoincKeyName;
 
+    /** @var string optional uuid column */
+    protected $uuidColumn;
+
     /** @var bool forbid updates to autoinc values */
     protected $protectAutoinc = true;
 
@@ -86,6 +92,9 @@ abstract class DbObject
 
     protected static $prefetchStats = array();
 
+    /** @var ?DbObjectStore */
+    protected static $dbObjectStore;
+
     /**
      * Constructor is not accessible and should not be overridden
      */
@@ -220,6 +229,11 @@ abstract class DbObject
         return $this;
     }
 
+    public static function setDbObjectStore(DbObjectStore $store)
+    {
+        self::$dbObjectStore = $store;
+    }
+
     /**
      * Getter
      *
@@ -307,6 +321,9 @@ abstract class DbObject
             $value = null;
         }
 
+        if (is_resource($value)) {
+            $value = stream_get_contents($value);
+        }
         $func = 'validate' . ucfirst($key);
         if (method_exists($this, $func) && $this->$func($value) !== true) {
             throw new InvalidArgumentException(sprintf(
@@ -530,6 +547,44 @@ abstract class DbObject
         return $this->autoincKeyName;
     }
 
+    /**
+     * @return ?string
+     */
+    public function getUuidColumn()
+    {
+        return $this->uuidColumn;
+    }
+
+    /**
+     * @return bool
+     */
+    public function hasUuidColumn()
+    {
+        return $this->uuidColumn !== null;
+    }
+
+    /**
+     * @return \Ramsey\Uuid\UuidInterface
+     */
+    public function getUniqueId()
+    {
+        if ($this->hasUuidColumn()) {
+            $binaryValue = $this->properties[$this->uuidColumn];
+            if (is_resource($binaryValue)) {
+                throw new RuntimeException('Properties contain binary UUID, probably a programming error');
+            }
+            if ($binaryValue === null) {
+                $uuid = Uuid::uuid4();
+                $this->reallySet($this->uuidColumn, $uuid->getBytes());
+                return $uuid;
+            }
+
+            return Uuid::fromBytes($binaryValue);
+        }
+
+        throw new InvalidArgumentException(sprintf('%s has no UUID column', $this->getTableName()));
+    }
+
     public function getKeyParams()
     {
         $params = array();
@@ -651,15 +706,22 @@ abstract class DbObject
     }
 
     /**
-     * @param object $row
+     * @param object|array $row
      * @param Db $db
      * @return self
      */
     public static function fromDbRow($row, Db $db)
     {
-        return (new static())
-            ->setConnection($db)
-            ->setDbProperties($row);
+        $self = (new static())->setConnection($db);
+        if (is_object($row)) {
+            return $self->setDbProperties((array) $row);
+        }
+
+        if (is_array($row)) {
+            return $self->setDbProperties($row);
+        }
+
+        throw new InvalidDataException('array or object', $row);
     }
 
     protected function setDbProperties($properties)
@@ -681,12 +743,27 @@ abstract class DbObject
             }
         }
 
+        $this->setBeingLoadedFromDb();
+        $this->onLoadFromDb();
+        return $this;
+    }
+
+    public function setBeingLoadedFromDb()
+    {
         $this->loadedFromDb = true;
         $this->loadedProperties = $this->properties;
         $this->hasBeenModified = false;
-        $this->modifiedProperties = array();
-        $this->onLoadFromDb();
-        return $this;
+        $this->modifiedProperties = [];
+    }
+
+    public function setLoadedProperty($key, $value)
+    {
+        if ($this->hasBeenLoadedFromDb()) {
+            $this->set($key, $value);
+            $this->loadedProperties[$key] = $this->get($key);
+        } else {
+            throw new RuntimeException('Cannot set loaded property for new object');
+        }
     }
 
     public function getOriginalProperties()
@@ -732,6 +809,7 @@ abstract class DbObject
             // Fake true, we might have manually set this to "modified"
             return true;
         }
+        $this->quoteBinaryProperties($properties);
 
         // TODO: Remember changed data for audit and log
         return $this->db->update(
@@ -755,28 +833,33 @@ abstract class DbObject
                 unset($properties[$this->autoincKeyName]);
             }
         }
-        // TODO: Remove this!
-        if ($this->connection->isPgsql()) {
-            foreach ($properties as $key => $value) {
-                if ($this->isBinaryColumn($key)) {
-                    $properties[$key] = Util::pgBinEscape($value);
-                }
-            }
+        if ($column = $this->getUuidColumn()) {
+            $properties[$column] = $this->getUniqueId()->getBytes();
         }
+        $this->quoteBinaryProperties($properties);
 
         return $this->db->insert($this->table, $properties);
     }
 
+    protected function quoteBinaryProperties(&$properties)
+    {
+        foreach ($properties as $key => $value) {
+            if ($this->isBinaryColumn($key)) {
+                $properties[$key] = $this->getConnection()->quoteBinary($value);
+            }
+        }
+    }
+
     protected function isBinaryColumn($column)
     {
-        return in_array($column, $this->binaryProperties);
+        return in_array($column, $this->binaryProperties) || $this->getUuidColumn() === $column;
     }
 
     /**
      * Store object to database
      *
      * @param  DbConnection $db
-     * @return bool Whether storing succeeded
+     * @return bool Whether storing succeeded. Always true, throws otherwise
      * @throws DuplicateKeyException
      */
     public function store(DbConnection $db = null)
@@ -804,7 +887,6 @@ abstract class DbObject
         try {
             if ($this->hasBeenLoadedFromDb()) {
                 if ($this->updateDb() !== false) {
-                    $result = true;
                     $this->onUpdate();
                 } else {
                     throw new RuntimeException(sprintf(
@@ -840,7 +922,6 @@ abstract class DbObject
                     }
                     // $this->log(sprintf('New %s "%s" has been stored', $table, $id));
                     $this->onInsert();
-                    $result = true;
                 } else {
                     throw new RuntimeException(sprintf(
                         'FAILED to store new %s "%s"',
@@ -859,13 +940,14 @@ abstract class DbObject
             ));
         }
 
-        $this->modifiedProperties = array();
+        // Hint: order is differs from setBeingLoadedFromDb() as of the onStore hook
+        $this->modifiedProperties = [];
         $this->hasBeenModified = false;
         $this->loadedProperties = $this->properties;
         $this->onStore();
         $this->loadedFromDb = true;
 
-        return $result;
+        return true;
     }
 
     /**
@@ -919,6 +1001,12 @@ abstract class DbObject
 
     public function createWhere()
     {
+        if ($this->hasUuidColumn() && $this->properties[$this->uuidColumn] !== null) {
+            return $this->db->quoteInto(
+                sprintf('%s = ?', $this->getUuidColumn()),
+                $this->connection->quoteBinary($this->getUniqueId()->getBytes())
+            );
+        }
         if ($id = $this->getAutoincId()) {
             if ($originalId = $this->getOriginalProperty($this->autoincKeyName)) {
                 return $this->db->quoteInto(
@@ -1147,8 +1235,15 @@ abstract class DbObject
             return $prefetched;
         }
 
-        /** @var DbObject $obj */
         $obj = new static;
+        if (self::$dbObjectStore !== null && $obj->hasUuidColumn()) {
+            $table = $obj->getTableName();
+            assert($connection instanceof Db);
+            $uuid = UuidLookup::requireUuidForKey($id, $table, $connection, self::$dbObjectStore->getBranch());
+
+            return self::$dbObjectStore->load($table, $uuid);
+        }
+
         $obj->setConnection($connection)
             ->set($obj->autoincKeyName, $id)
             ->loadFromDb();
@@ -1167,9 +1262,17 @@ abstract class DbObject
         if ($prefetched = static::getPrefetched($id)) {
             return $prefetched;
         }
-
         /** @var DbObject $obj */
         $obj = new static;
+
+        if (self::$dbObjectStore !== null && $obj->hasUuidColumn()) {
+            $table = $obj->getTableName();
+            assert($connection instanceof Db);
+            $uuid = UuidLookup::requireUuidForKey($id, $table, $connection, self::$dbObjectStore->getBranch());
+
+            return self::$dbObjectStore->load($table, $uuid);
+        }
+
         $obj->setConnection($connection)->setKey($id)->loadFromDb();
 
         return $obj;
@@ -1268,10 +1371,91 @@ abstract class DbObject
 
         /** @var DbObject $obj */
         $obj = new static;
+        if (self::$dbObjectStore !== null) {
+            $table = $obj->getTableName();
+            assert($connection instanceof Db);
+            $uuid = UuidLookup::findUuidForKey($id, $table, $connection, self::$dbObjectStore->getBranch());
+            if ($uuid) {
+                return self::$dbObjectStore->exists($table, $uuid);
+            }
+
+            return false;
+        }
+
         $obj->setConnection($connection)->setKey($id);
         return $obj->existsInDb();
     }
 
+    public static function uniqueIdExists(UuidInterface $uuid, DbConnection $connection)
+    {
+        $db = $connection->getDbAdapter();
+        $obj = new static;
+        $column = $obj->getUuidColumn();
+        $query = $db->select()
+            ->from($obj->getTableName(), $column)
+            ->where("$column = ?", $connection->quoteBinary($uuid->getBytes()));
+
+        $result = $db->fetchRow($query);
+
+        return $result !== false;
+    }
+
+    public static function requireWithUniqueId(UuidInterface $uuid, DbConnection $connection)
+    {
+        if ($object = static::loadWithUniqueId($uuid, $connection)) {
+            return $object;
+        }
+
+        throw new NotFoundError(sprintf(
+            'No %s with UUID=%s has been found',
+            (new static)->getTableName(),
+            $uuid->toString()
+        ));
+    }
+
+    public static function loadWithUniqueId(UuidInterface $uuid, DbConnection $connection)
+    {
+        $db = $connection->getDbAdapter();
+        $obj = new static;
+
+        if (self::$dbObjectStore !== null) {
+            $table = $obj->getTableName();
+            assert($connection instanceof Db);
+            return self::$dbObjectStore->load($table, $uuid);
+        }
+
+        $query = $db->select()
+            ->from($obj->getTableName())
+            ->where($obj->getUuidColumn() . ' = ?', $connection->quoteBinary($uuid->getBytes()));
+
+        $result = $db->fetchRow($query);
+
+        if ($result) {
+            return $obj->setConnection($connection)->setDbProperties($result);
+        }
+
+        return null;
+    }
+
+    public function setUniqueId(UuidInterface $uuid)
+    {
+        if ($column = $this->getUuidColumn()) {
+            $binary = $uuid->getBytes();
+            $current = $this->get($column);
+            if ($current === null) {
+                $this->set($column, $binary);
+            } else {
+                if ($current !== $binary) {
+                    throw new RuntimeException(sprintf(
+                        'Changing the UUID (from %s to %s) is not allowed',
+                        Uuid::fromBytes($current)->toString(),
+                        Uuid::fromBytes($binary)->toString()
+                    ));
+                }
+            }
+        }
+    }
+
     public function __destruct()
     {
         unset($this->db);
