@@ -17,7 +17,7 @@ class DirectorActivityLog extends DbObject
 
     protected $autoincKeyName = 'id';
 
-    protected $defaultProperties = array(
+    protected $defaultProperties = [
         'id'              => null,
         'object_name'     => null,
         'action_name'     => null,
@@ -28,7 +28,12 @@ class DirectorActivityLog extends DbObject
         'change_time'     => null,
         'checksum'        => null,
         'parent_checksum' => null,
-    );
+    ];
+
+    protected $binaryProperties = [
+        'checksum',
+        'parent_checksum'
+    ];
 
     /**
      * @param $name
@@ -82,7 +87,7 @@ class DirectorActivityLog extends DbObject
     public static function loadLatest(Db $connection)
     {
         $db = $connection->getDbAdapter();
-        $query = $db->select()->from('director_activity_log', array('id' => 'MAX(id)'));
+        $query = $db->select()->from('director_activity_log', ['id' => 'MAX(id)']);
 
         return static::load($db->fetchOne($query), $connection);
     }
@@ -105,7 +110,7 @@ class DirectorActivityLog extends DbObject
         );
 
         $data['checksum'] = sha1(json_encode($data), true);
-        $data['parent_checksum'] = Util::hex2binary($data['parent_checksum']);
+        $data['parent_checksum'] = hex2bin($data['parent_checksum']);
 
         static::audit($db, array(
             'action'      => 'create',
@@ -136,7 +141,7 @@ class DirectorActivityLog extends DbObject
         );
 
         $data['checksum'] = sha1(json_encode($data), true);
-        $data['parent_checksum'] = Util::hex2binary($data['parent_checksum']);
+        $data['parent_checksum'] = hex2bin($data['parent_checksum']);
 
         static::audit($db, array(
             'action'      => 'modify',
@@ -166,7 +171,7 @@ class DirectorActivityLog extends DbObject
         );
 
         $data['checksum'] = sha1(json_encode($data), true);
-        $data['parent_checksum'] = Util::hex2binary($data['parent_checksum']);
+        $data['parent_checksum'] = hex2bin($data['parent_checksum']);
 
         static::audit($db, array(
             'action'      => 'remove',
