@@ -30,11 +30,13 @@ class IcingaArguments implements Iterator, Countable, IcingaConfigRenderer
         $this->object = $object;
     }
 
+    #[\ReturnTypeWillChange]
     public function count()
     {
         return count($this->arguments);
     }
 
+    #[\ReturnTypeWillChange]
     public function rewind()
     {
         $this->position = 0;
@@ -45,6 +47,7 @@ class IcingaArguments implements Iterator, Countable, IcingaConfigRenderer
         return $this->modified;
     }
 
+    #[\ReturnTypeWillChange]
     public function current()
     {
         if (! $this->valid()) {
@@ -54,16 +57,19 @@ class IcingaArguments implements Iterator, Countable, IcingaConfigRenderer
         return $this->arguments[$this->idx[$this->position]];
     }
 
+    #[\ReturnTypeWillChange]
     public function key()
     {
         return $this->idx[$this->position];
     }
 
+    #[\ReturnTypeWillChange]
     public function next()
     {
         ++$this->position;
     }
 
+    #[\ReturnTypeWillChange]
     public function valid()
     {
         return array_key_exists($this->position, $this->idx);
@@ -84,6 +90,10 @@ class IcingaArguments implements Iterator, Countable, IcingaConfigRenderer
 
     public function set($key, $value)
     {
+        if ($value === null) {
+            return $this->remove($key);
+        }
+
         if ($value instanceof IcingaCommandArgument) {
             $argument = $value;
         } else {
@@ -141,13 +151,19 @@ class IcingaArguments implements Iterator, Countable, IcingaConfigRenderer
                 }
             }
             if (property_exists($value, 'type')) {
+                // argument is directly set as function, no further properties
                 if ($value->type === 'Function') {
                     $attrs['argument_value'] = '/* Unable to fetch function body through API */';
                     $attrs['argument_format'] = 'expression';
                 }
             } elseif (property_exists($value, 'value')) {
+                // argument is a dictionary with further settings
                 if (is_object($value->value)) {
-                    if ($value->value->type === 'Function') {
+                    if ($value->value->type === 'Function' && property_exists($value->value, 'body')) {
+                        // likely an export from Baskets that contains the actual function body
+                        $attrs['argument_value'] = $value->value->body;
+                        $attrs['argument_format'] = 'expression';
+                    } elseif ($value->value->type === 'Function') {
                         $attrs['argument_value'] = '/* Unable to fetch function body through API */';
                         $attrs['argument_format'] = 'expression';
                     } else {
@@ -174,28 +190,22 @@ class IcingaArguments implements Iterator, Countable, IcingaConfigRenderer
             }
         }
 
-        if (array_key_exists('set_if', $attrs) && is_object($attrs['set_if'])) {
-            if ($attrs['set_if']->type === 'Function') {
+        if (array_key_exists('set_if', $attrs)) {
+            if (is_object($attrs['set_if']) && $attrs['set_if']->type === 'Function') {
                 $attrs['set_if'] = '/* Unable to fetch function body through API */';
                 $attrs['set_if_format'] = 'expression';
+            } elseif (property_exists($value, 'set_if_format')) {
+                if (in_array($value->set_if_format, ['string', 'expression', 'json'])) {
+                    $attrs['set_if_format'] = $value->set_if_format;
+                }
             }
         }
 
         return $attrs;
     }
 
-    // TODO -> UNFINISHED!!!
     public function setArguments($arguments)
     {
-        if (empty($arguments)) {
-            if (count($this->arguments)) {
-                $this->arguments = [];
-                $this->modified = true;
-            }
-
-            return $this;
-        }
-
         $arguments = (array) $arguments;
 
         foreach ($arguments as $arg => $val) {
@@ -341,6 +351,15 @@ class IcingaArguments implements Iterator, Countable, IcingaConfigRenderer
         return $arguments->loadFromDb();
     }
 
+    public function setBeingLoadedFromDb()
+    {
+        foreach ($this->arguments as $argument) {
+            $argument->setBeingLoadedFromDb();
+        }
+        $this->refreshIndex();
+        $this->cloneStored();
+    }
+
     /**
      * @return $this
      * @throws \Icinga\Module\Director\Exception\DuplicateKeyException
@@ -353,13 +372,22 @@ class IcingaArguments implements Iterator, Countable, IcingaConfigRenderer
             if ($argument->shouldBeRemoved()) {
                 $deleted[] = $key;
             } else {
-                $argument->set('command_id', $this->object->get('id'));
-                $argument->store($db);
+                if ($argument->hasBeenModified()) {
+                    if ($argument->hasBeenLoadedFromDb()) {
+                        $argument->setLoadedProperty('command_id', $this->object->get('id'));
+                    } else {
+                        $argument->set('command_id', $this->object->get('id'));
+                    }
+                    $argument->store($db);
+                }
             }
         }
 
         foreach ($deleted as $key) {
-            $this->arguments[$key]->delete();
+            $argument = $this->arguments[$key];
+            $argument->setLoadedProperty('command_id', $this->object->get('id'));
+            $argument->setConnection($this->object->getConnection());
+            $argument->delete();
             unset($this->arguments[$key]);
         }
 
