@@ -6,11 +6,13 @@ use Exception;
 use Icinga\Data\Filter\Filter;
 use Icinga\Module\Director\Db;
 use Icinga\Module\Director\Db\Cache\PrefetchCache;
+use Icinga\Module\Director\Db\DbUtil;
 use Icinga\Module\Director\DirectorObject\Automation\ExportInterface;
 use Icinga\Module\Director\Exception\DuplicateKeyException;
 use Icinga\Module\Director\IcingaConfig\IcingaConfig;
 use Icinga\Module\Director\Resolver\HostServiceBlacklist;
 use InvalidArgumentException;
+use Ramsey\Uuid\Uuid;
 use RuntimeException;
 
 class IcingaServiceSet extends IcingaObject implements ExportInterface
@@ -19,6 +21,7 @@ class IcingaServiceSet extends IcingaObject implements ExportInterface
 
     protected $defaultProperties = array(
         'id'                    => null,
+        'uuid'                  => null,
         'host_id'               => null,
         'object_name'           => null,
         'object_type'           => null,
@@ -26,6 +29,8 @@ class IcingaServiceSet extends IcingaObject implements ExportInterface
         'assign_filter'         => null,
     );
 
+    protected $uuidColumn = 'uuid';
+
     protected $keyName = array('host_id', 'object_name');
 
     protected $supportsImports = true;
@@ -105,20 +110,16 @@ class IcingaServiceSet extends IcingaObject implements ExportInterface
         if ($set->get('id') === null) {
             return array();
         }
-
         $connection = $this->getConnection();
         $db = $this->getDb();
-        $ids = $db->fetchCol(
-            $db->select()->from('icinga_service', 'id')
+        $uuids = $db->fetchCol(
+            $db->select()->from('icinga_service', 'uuid')
                 ->where('service_set_id = ?', $set->get('id'))
         );
 
         $services = array();
-        foreach ($ids as $id) {
-            $service = IcingaService::load(array(
-                'id' => $id,
-                'object_type' => 'template'
-            ), $connection);
+        foreach ($uuids as $uuid) {
+            $service = IcingaService::loadWithUniqueId(Uuid::fromBytes(DbUtil::binaryResult($uuid)), $connection);
             $service->set('service_set', null);
 
             $services[$service->getObjectName()] = $service;
@@ -218,11 +219,13 @@ class IcingaServiceSet extends IcingaObject implements ExportInterface
             's.*'
         )->where('service_set_id = ?', $setId);
         $existingServices = IcingaService::loadAll($db, $sQuery, 'object_name');
+        $serviceNames = [];
         foreach ($services as $service) {
             if (isset($service->fields)) {
                 unset($service->fields);
             }
             $name = $service->object_name;
+            $serviceNames[] = $name;
             if (isset($existingServices[$name])) {
                 $existing = $existingServices[$name];
                 $existing->setProperties((array) $service);
@@ -237,6 +240,12 @@ class IcingaServiceSet extends IcingaObject implements ExportInterface
             }
         }
 
+        foreach ($existingServices as $existing) {
+            if (!in_array($existing->getObjectName(), $serviceNames)) {
+                $existing->delete();
+            }
+        }
+
         return $object;
     }
 
