@@ -4,7 +4,9 @@ namespace Icinga\Module\Director\ProvidedHook\Monitoring;
 
 use Exception;
 use Icinga\Application\Config;
+use Icinga\Authentication\Auth;
 use Icinga\Module\Director\Db;
+use Icinga\Module\Director\Monitoring;
 use Icinga\Module\Director\Objects\IcingaHost;
 use Icinga\Module\Director\Util;
 use Icinga\Module\Monitoring\Hook\HostActionsHook;
@@ -37,7 +39,19 @@ class HostActions extends HostActionsHook
             );
         }
 
+        $allowEdit = false;
         if (Util::hasPermission('director/hosts') && IcingaHost::exists($hostname, $db)) {
+            $allowEdit = true;
+        }
+        $auth = Auth::getInstance();
+        if (Util::hasPermission('director/monitoring/hosts')) {
+            $monitoring = new Monitoring();
+            if ($monitoring->isAvailable() && $monitoring->authCanEditHost($auth, $hostname)) {
+                $allowEdit = IcingaHost::exists($hostname, $db);
+            }
+        }
+
+        if ($allowEdit) {
             $actions[mt('director', 'Modify')] = Url::fromPath(
                 'director/host/edit',
                 array('name' => $hostname)
