@@ -56,12 +56,11 @@ class HostgroupRestriction extends ObjectRestriction
             return true;
         }
 
-        if (! $host->hasBeenLoadedFromDb()) {
-            if ($host->hasModifiedGroups()) {
-                foreach ($this->listRestrictedHostgroups() as $group) {
-                    if ($host->hasGroup($group)) {
-                        return true;
-                    }
+        // Hint: branched hosts have no id
+        if (! $host->hasBeenLoadedFromDb() || $host->hasModifiedGroups() || $host->get('id') === null) {
+            foreach ($this->listRestrictedHostgroups() as $group) {
+                if ($host->hasGroup($group)) {
+                    return true;
                 }
             }
 
