@@ -2,11 +2,11 @@
 
 namespace Icinga\Module\Director\Web\Widget;
 
+use gipfl\Json\JsonString;
 use ipl\Html\HtmlDocument;
 use ipl\Html\HtmlElement;
 use Icinga\Date\DateFormatter;
 use Icinga\Exception\ProgrammingError;
-use Icinga\Module\Director\ConfigDiff;
 use Icinga\Module\Director\Db;
 use Icinga\Module\Director\Forms\RestoreObjectForm;
 use Icinga\Module\Director\IcingaConfig\IcingaConfig;
@@ -125,12 +125,12 @@ class ActivityLogInfo extends HtmlDocument
         $this->add($this->getInfoTable());
         if ($tabName === 'old') {
             // $title = sprintf('%s former config', $this->entry->object_name);
-            $diffs = $this->getConfigDiffs($this->oldConfig(), $this->emptyConfig());
+            $diffs = IcingaConfigDiff::getDiffs($this->oldConfig(), $this->emptyConfig());
         } elseif ($tabName === 'new') {
             // $title = sprintf('%s new config', $this->entry->object_name);
-            $diffs = $this->getConfigDiffs($this->emptyConfig(), $this->newConfig());
+            $diffs = IcingaConfigDiff::getDiffs($this->emptyConfig(), $this->newConfig());
         } else {
-            $diffs = $this->getConfigDiffs($this->oldConfig(), $this->newConfig());
+            $diffs = IcingaConfigDiff::getDiffs($this->oldConfig(), $this->newConfig());
         }
 
         $this->addDiffs($diffs);
@@ -155,7 +155,7 @@ class ActivityLogInfo extends HtmlDocument
     }
 
     /**
-     * @return $this
+     * @return RestoreObjectForm
      * @throws \Icinga\Exception\IcingaException
      */
     protected function getRestoreForm()
@@ -221,7 +221,7 @@ class ActivityLogInfo extends HtmlDocument
     {
         if ($this->oldProperties === null) {
             if (property_exists($this->entry, 'old_properties')) {
-                $this->oldProperties = json_decode($this->entry->old_properties);
+                $this->oldProperties = JsonString::decodeOptional($this->entry->old_properties);
             }
             if ($this->oldProperties === null) {
                 $this->oldProperties = new \stdClass;
@@ -235,7 +235,7 @@ class ActivityLogInfo extends HtmlDocument
     {
         if ($this->newProperties === null) {
             if (property_exists($this->entry, 'new_properties')) {
-                $this->newProperties = json_decode($this->entry->new_properties);
+                $this->newProperties = JsonString::decodeOptional($this->entry->new_properties);
             }
             if ($this->newProperties === null) {
                 $this->newProperties = new \stdClass;
@@ -397,41 +397,6 @@ class ActivityLogInfo extends HtmlDocument
         return $tabs;
     }
 
-    /**
-     * @param IcingaConfig $oldConfig
-     * @param IcingaConfig $newConfig
-     * @return ConfigDiff[]
-     */
-    protected function getConfigDiffs(IcingaConfig $oldConfig, IcingaConfig $newConfig)
-    {
-        $oldFileNames = $oldConfig->getFileNames();
-        $newFileNames = $newConfig->getFileNames();
-
-        $fileNames = array_merge($oldFileNames, $newFileNames);
-
-        $diffs = [];
-        foreach ($fileNames as $filename) {
-            if (in_array($filename, $oldFileNames)) {
-                $left = $oldConfig->getFile($filename)->getContent();
-            } else {
-                $left = '';
-            }
-
-            if (in_array($filename, $newFileNames)) {
-                $right = $newConfig->getFile($filename)->getContent();
-            } else {
-                $right = '';
-            }
-            if ($left === $right) {
-                continue;
-            }
-
-            $diffs[$filename] = ConfigDiff::create($left, $right);
-        }
-
-        return $diffs;
-    }
-
     /**
      * @return IcingaObject
      * @throws \Icinga\Exception\IcingaException
@@ -557,6 +522,13 @@ class ActivityLogInfo extends HtmlDocument
             );
         }
 
+        if ($comment = $this->getOptionalRangeComment()) {
+            $table->addNameValueRow(
+                $this->translate('Remark'),
+                $comment
+            );
+        }
+
         if ($this->hasBeenEnabled()) {
             $table->addNameValueRow(
                 $this->translate('Rendering'),
@@ -619,6 +591,21 @@ class ActivityLogInfo extends HtmlDocument
         return sprintf($msg, $this->typeName, $this->entry->object_name);
     }
 
+    protected function getOptionalRangeComment()
+    {
+        if ($this->id) {
+            $db = $this->db->getDbAdapter();
+            return $db->fetchOne(
+                $db->select()
+                    ->from('director_activity_log_remark', 'remark')
+                    ->where('first_related_activity <= ?', $this->id)
+                    ->where('last_related_activity >= ?', $this->id)
+            );
+        }
+
+        return null;
+    }
+
     /**
      * @param $type
      * @param $props
