@@ -36,7 +36,8 @@ CREATE TYPE enum_sync_rule_object_type AS ENUM(
   'notification',
   'dependency'
 );
-CREATE TYPE enum_sync_rule_update_policy AS ENUM('merge', 'override', 'ignore');
+CREATE TYPE enum_sync_rule_update_policy AS ENUM('merge', 'override', 'ignore', 'update-only');
+CREATE TYPE enum_sync_rule_purge_action AS ENUM('delete', 'disable');
 CREATE TYPE enum_sync_property_merge_policy AS ENUM('override', 'merge');
 CREATE TYPE enum_sync_state AS ENUM(
     'unknown',
@@ -46,7 +47,7 @@ CREATE TYPE enum_sync_state AS ENUM(
 );
 CREATE TYPE enum_host_service AS ENUM('host', 'service');
 CREATE TYPE enum_owner_type AS ENUM('user', 'usergroup', 'role');
-
+CREATE DOMAIN d_smallint AS integer CHECK (VALUE >= 0) CHECK (VALUE < 65536);
 
 CREATE OR REPLACE FUNCTION unix_timestamp(timestamp with time zone) RETURNS bigint AS '
         SELECT EXTRACT(EPOCH FROM $1)::bigint AS result
@@ -96,6 +97,26 @@ CREATE INDEX activity_log_author ON director_activity_log (author);
 COMMENT ON COLUMN director_activity_log.old_properties IS 'Property hash, JSON';
 COMMENT ON COLUMN director_activity_log.new_properties IS 'Property hash, JSON';
 
+CREATE TABLE director_activity_log_remark (
+  first_related_activity bigint NOT NULL,
+  last_related_activity bigint NOT NULL,
+  remark TEXT NOT NULL,
+  PRIMARY KEY (first_related_activity, last_related_activity),
+  CONSTRAINT activity_log_remark_begin
+    FOREIGN KEY (first_related_activity)
+      REFERENCES director_activity_log (id)
+      ON DELETE CASCADE
+      ON UPDATE CASCADE,
+  CONSTRAINT activity_log_remark_end
+    FOREIGN KEY (last_related_activity)
+      REFERENCES director_activity_log (id)
+      ON DELETE CASCADE
+      ON UPDATE CASCADE
+);
+
+CREATE INDEX first_related_activity ON director_activity_log_remark (first_related_activity);
+CREATE INDEX last_related_activity ON director_activity_log_remark (last_related_activity);
+
 
 CREATE TABLE director_basket (
   uuid bytea CHECK(LENGTH(uuid) = 16) NOT NULL,
@@ -248,18 +269,35 @@ CREATE TABLE director_datalist_entry (
 CREATE INDEX datalist_entry_datalist ON director_datalist_entry (list_id);
 
 
+CREATE TABLE director_datafield_category (
+  id serial,
+  category_name character varying(255) NOT NULL,
+  description text DEFAULT NULL,
+  PRIMARY KEY (id)
+);
+
+CREATE UNIQUE INDEX datafield_category_name ON director_datafield_category (category_name);
+
+
 CREATE TABLE director_datafield (
   id serial,
+  category_id integer DEFAULT NULL,
   varname character varying(64) NOT NULL,
   caption character varying(255) NOT NULL,
   description text DEFAULT NULL,
   datatype character varying(255) NOT NULL,
 -- datatype_param? multiple ones?
   format enum_property_format,
-  PRIMARY KEY (id)
+  PRIMARY KEY (id),
+  CONSTRAINT director_datafield_category
+    FOREIGN KEY (category_id)
+    REFERENCES director_datafield_category (id)
+    ON DELETE RESTRICT
+    ON UPDATE CASCADE
 );
 
 CREATE INDEX search_idx ON director_datafield (varname);
+CREATE INDEX datafield_category ON director_datafield (category_id);
 
 
 CREATE TABLE director_datafield_setting (
@@ -293,6 +331,7 @@ CREATE TABLE director_setting (
 
 CREATE TABLE icinga_zone (
   id serial,
+  uuid bytea UNIQUE CHECK(LENGTH(uuid) = 16),
   parent_id integer DEFAULT NULL,
   object_name character varying(255) NOT NULL UNIQUE,
   object_type enum_object_type_all NOT NULL,
@@ -307,6 +346,7 @@ CREATE TABLE icinga_zone (
 );
 
 CREATE INDEX zone_parent ON icinga_zone (parent_id);
+CREATE UNIQUE INDEX zone_uuid ON icinga_zone (uuid);
 
 
 CREATE TABLE icinga_zone_inheritance (
@@ -333,6 +373,7 @@ CREATE INDEX zone_inheritance_zone_parent ON icinga_zone_inheritance (parent_zon
 
 CREATE TABLE icinga_timeperiod (
   id serial,
+  uuid bytea UNIQUE CHECK(LENGTH(uuid) = 16),
   object_name character varying(255) NOT NULL,
   display_name character varying(255) DEFAULT NULL,
   update_method character varying(64) DEFAULT NULL,
@@ -348,6 +389,7 @@ CREATE TABLE icinga_timeperiod (
     ON UPDATE CASCADE
 );
 
+CREATE UNIQUE INDEX timeperiod_uuid ON icinga_timeperiod (uuid);
 CREATE UNIQUE INDEX timeperiod_object_name ON icinga_timeperiod (object_name, zone_id);
 CREATE INDEX timeperiod_zone ON icinga_timeperiod (zone_id);
 COMMENT ON COLUMN icinga_timeperiod.update_method IS 'Usually LegacyTimePeriod';
@@ -435,6 +477,7 @@ CREATE INDEX director_job_setting_job ON director_job_setting (job_id);
 
 CREATE TABLE icinga_command (
   id serial,
+  uuid bytea UNIQUE CHECK(LENGTH(uuid) = 16),
   object_name character varying(255) NOT NULL,
   object_type enum_object_type_all NOT NULL,
   disabled enum_boolean NOT NULL DEFAULT 'n',
@@ -452,6 +495,7 @@ CREATE TABLE icinga_command (
     ON UPDATE CASCADE
 );
 
+CREATE UNIQUE INDEX command_uuid ON icinga_command (uuid);
 CREATE UNIQUE INDEX command_object_name ON icinga_command (object_name);
 CREATE INDEX command_zone ON icinga_command (zone_id);
 COMMENT ON COLUMN icinga_command.object_type IS 'external_object is an attempt to work with existing commands';
@@ -549,6 +593,7 @@ CREATE INDEX command_var_checksum ON icinga_command_var (checksum);
 
 CREATE TABLE icinga_apiuser (
   id BIGSERIAL,
+  uuid bytea UNIQUE CHECK(LENGTH(uuid) = 16),
   object_name CHARACTER VARYING(255) NOT NULL,
   object_type enum_object_type_all NOT NULL,
   disabled enum_boolean NOT NULL DEFAULT 'n',
@@ -558,17 +603,19 @@ CREATE TABLE icinga_apiuser (
   PRIMARY KEY (id)
 );
 
+CREATE UNIQUE INDEX apiuser_uuid ON icinga_apiuser (uuid);
 COMMENT ON COLUMN icinga_apiuser.permissions IS 'JSON-encoded permissions';
 
 
 CREATE TABLE icinga_endpoint (
   id serial,
+  uuid bytea UNIQUE CHECK(LENGTH(uuid) = 16),
   zone_id integer DEFAULT NULL,
   object_name character varying(255) NOT NULL,
   object_type enum_object_type_all NOT NULL,
   disabled enum_boolean NOT NULL DEFAULT 'n',
   host character varying(255) DEFAULT NULL,
-  port smallint DEFAULT NULL,
+  port d_smallint DEFAULT NULL,
   log_duration character varying(32) DEFAULT NULL,
   apiuser_id INTEGER DEFAULT NULL,
   PRIMARY KEY (id),
@@ -584,6 +631,7 @@ CREATE TABLE icinga_endpoint (
     ON UPDATE CASCADE
 );
 
+CREATE UNIQUE INDEX endpoint_uuid ON icinga_endpoint (uuid);
 CREATE UNIQUE INDEX endpoint_object_name ON icinga_endpoint (object_name);
 CREATE INDEX endpoint_zone ON icinga_endpoint (zone_id);
 COMMENT ON COLUMN icinga_endpoint.host IS 'IP address / hostname of remote node';
@@ -629,6 +677,7 @@ CREATE INDEX host_template_choice_required_template ON icinga_host_template_choi
 
 CREATE TABLE icinga_host (
   id serial,
+  uuid bytea UNIQUE CHECK(LENGTH(uuid) = 16),
   object_name character varying(255) NOT NULL,
   object_type enum_object_type_all NOT NULL,
   disabled enum_boolean NOT NULL DEFAULT 'n',
@@ -661,6 +710,7 @@ CREATE TABLE icinga_host (
   has_agent enum_boolean DEFAULT NULL,
   master_should_connect enum_boolean DEFAULT NULL,
   accept_config enum_boolean DEFAULT NULL,
+  custom_endpoint_name character varying(255) DEFAULT NULL,
   api_key character varying(40) DEFAULT NULL,
   template_choice_id int DEFAULT NULL,
   PRIMARY KEY (id),
@@ -697,6 +747,7 @@ CREATE TABLE icinga_host (
 );
 
 
+CREATE UNIQUE INDEX host_uuid ON icinga_host (uuid);
 CREATE UNIQUE INDEX object_name_host ON icinga_host (object_name, zone_id);
 CREATE UNIQUE INDEX host_api_key ON icinga_host (api_key);
 CREATE INDEX host_zone ON icinga_host (zone_id);
@@ -782,6 +833,7 @@ ALTER TABLE icinga_host_template_choice
 
 CREATE TABLE icinga_service_set (
   id serial,
+  uuid bytea UNIQUE CHECK(LENGTH(uuid) = 16),
   host_id integer DEFAULT NULL,
   object_name character varying(128) NOT NULL,
   object_type enum_object_type_all NOT NULL,
@@ -797,6 +849,7 @@ CREATE TABLE icinga_service_set (
 
 CREATE UNIQUE INDEX service_set_name ON icinga_service_set (object_name, host_id);
 CREATE INDEX service_set_host ON icinga_service_set (host_id);
+CREATE UNIQUE INDEX service_set_uuid ON icinga_service_set (uuid);
 
 
 CREATE TABLE icinga_service_template_choice (
@@ -816,6 +869,7 @@ CREATE INDEX service_template_choice_required_template ON icinga_service_templat
 
 CREATE TABLE icinga_service (
   id serial,
+  uuid bytea UNIQUE CHECK(LENGTH(uuid) = 16),
   object_name character varying(255) NOT NULL,
   object_type enum_object_type_all NOT NULL,
   disabled enum_boolean DEFAULT 'n',
@@ -894,6 +948,7 @@ CREATE TABLE icinga_service (
     ON UPDATE CASCADE
 );
 
+CREATE UNIQUE INDEX service_uuid ON icinga_service (uuid);
 CREATE INDEX service_zone ON icinga_service (zone_id);
 CREATE INDEX service_timeperiod ON icinga_service (check_period_id);
 CREATE INDEX service_check_command ON icinga_service (check_command_id);
@@ -1058,6 +1113,7 @@ CREATE INDEX service_set_var_checksum ON icinga_service_set_var (checksum);
 
 CREATE TABLE icinga_hostgroup (
   id serial,
+  uuid bytea UNIQUE CHECK(LENGTH(uuid) = 16),
   object_name character varying(255) NOT NULL,
   object_type enum_object_type_all NOT NULL,
   disabled enum_boolean NOT NULL DEFAULT 'n',
@@ -1066,6 +1122,7 @@ CREATE TABLE icinga_hostgroup (
   PRIMARY KEY (id)
 );
 
+CREATE UNIQUE INDEX hostgroup_uuid ON icinga_hostgroup (uuid);
 CREATE UNIQUE INDEX hostgroup_object_name ON icinga_hostgroup (object_name);
 CREATE INDEX hostgroup_search_idx ON icinga_hostgroup (display_name);
 
@@ -1095,6 +1152,7 @@ CREATE INDEX hostgroup_inheritance_hostgroup_parent ON icinga_hostgroup_inherita
 
 CREATE TABLE icinga_servicegroup (
   id serial,
+  uuid bytea UNIQUE CHECK(LENGTH(uuid) = 16),
   object_name character varying(255) DEFAULT NULL,
   object_type enum_object_type_all NOT NULL,
   disabled enum_boolean NOT NULL DEFAULT 'n',
@@ -1103,6 +1161,7 @@ CREATE TABLE icinga_servicegroup (
   PRIMARY KEY (id)
 );
 
+CREATE UNIQUE INDEX servicegroup_uuid ON icinga_servicegroup (uuid);
 CREATE UNIQUE INDEX servicegroup_object_name ON icinga_servicegroup (object_name);
 CREATE INDEX servicegroup_search_idx ON icinga_servicegroup (display_name);
 
@@ -1230,6 +1289,7 @@ CREATE INDEX hostgroup_parent_parent ON icinga_hostgroup_parent (parent_hostgrou
 
 CREATE TABLE icinga_user (
   id serial,
+  uuid bytea UNIQUE CHECK(LENGTH(uuid) = 16),
   object_name character varying(255) DEFAULT NULL,
   object_type enum_object_type_all NOT NULL,
   disabled enum_boolean NOT NULL DEFAULT 'n',
@@ -1252,6 +1312,7 @@ CREATE TABLE icinga_user (
     ON UPDATE CASCADE
 );
 
+CREATE UNIQUE INDEX user_uuid ON icinga_user (uuid);
 CREATE UNIQUE INDEX user_object_name ON icinga_user (object_name, zone_id);
 CREATE INDEX user_zone ON icinga_user (zone_id);
 
@@ -1355,6 +1416,7 @@ COMMENT ON COLUMN icinga_user_field.user_id IS 'Makes only sense for templates';
 
 CREATE TABLE icinga_usergroup (
   id serial,
+  uuid bytea UNIQUE CHECK(LENGTH(uuid) = 16),
   object_name character varying(255) NOT NULL,
   object_type enum_object_type_all NOT NULL,
   disabled enum_boolean NOT NULL DEFAULT 'n',
@@ -1368,6 +1430,7 @@ CREATE TABLE icinga_usergroup (
       ON UPDATE CASCADE
 );
 
+CREATE UNIQUE INDEX usergroup_uuid ON icinga_usergroup (uuid);
 CREATE UNIQUE INDEX usergroup_search_idx ON icinga_usergroup (display_name);
 CREATE INDEX usergroup_object_name ON icinga_usergroup (object_name);
 CREATE INDEX usergroup_zone ON icinga_usergroup (zone_id);
@@ -1437,6 +1500,7 @@ CREATE INDEX usergroup_parent_parent ON icinga_usergroup_parent (parent_usergrou
 
 CREATE TABLE icinga_notification (
   id serial,
+  uuid bytea UNIQUE CHECK(LENGTH(uuid) = 16),
   object_name CHARACTER VARYING(255) DEFAULT NULL,
   object_type enum_object_type_all NOT NULL,
   disabled enum_boolean NOT NULL DEFAULT 'n',
@@ -1478,6 +1542,8 @@ CREATE TABLE icinga_notification (
     ON UPDATE CASCADE
 );
 
+CREATE UNIQUE INDEX notification_uuid ON icinga_notification (uuid);
+
 
 CREATE TABLE icinga_notification_user (
   notification_id integer NOT NULL,
@@ -1673,6 +1739,7 @@ CREATE TABLE sync_rule (
   object_type enum_sync_rule_object_type NOT NULL,
   update_policy enum_sync_rule_update_policy NOT NULL,
   purge_existing enum_boolean NOT NULL DEFAULT 'n',
+  purge_action enum_sync_rule_purge_action NULL DEFAULT NULL,
   filter_expression text DEFAULT NULL,
   sync_state enum_sync_state NOT NULL DEFAULT 'unknown',
   last_error_message text NULL DEFAULT NULL,
@@ -1985,6 +2052,7 @@ CREATE INDEX user_resolved_var_schecksum ON icinga_user_resolved_var (checksum);
 
 CREATE TABLE icinga_dependency (
   id serial,
+  uuid bytea UNIQUE CHECK(LENGTH(uuid) = 16),
   object_name character varying(255) NOT NULL,
   object_type enum_object_type_all NOT NULL,
   disabled enum_boolean DEFAULT 'n',
@@ -2034,6 +2102,7 @@ CREATE TABLE icinga_dependency (
     ON UPDATE CASCADE
 );
 
+CREATE UNIQUE INDEX dependency_uuid ON icinga_dependency (uuid);
 CREATE INDEX dependency_parent_host ON icinga_dependency (parent_host_id);
 CREATE INDEX dependency_parent_service ON icinga_dependency (parent_service_id);
 CREATE INDEX dependency_child_host ON icinga_dependency (child_host_id);
@@ -2114,6 +2183,7 @@ CREATE TABLE icinga_timeperiod_exclude (
 
 CREATE TABLE icinga_scheduled_downtime (
   id serial,
+  uuid bytea UNIQUE CHECK(LENGTH(uuid) = 16),
   object_name character varying(255) NOT NULL,
   zone_id integer DEFAULT NULL,
   object_type enum_object_type_all NOT NULL,
@@ -2133,6 +2203,7 @@ CREATE TABLE icinga_scheduled_downtime (
     ON UPDATE CASCADE
 );
 
+CREATE UNIQUE INDEX scheduled_downtime_uuid ON icinga_scheduled_downtime (uuid);
 CREATE UNIQUE INDEX scheduled_downtime_object_name ON icinga_scheduled_downtime (object_name);
 CREATE INDEX scheduled_downtime_zone ON icinga_scheduled_downtime (zone_id);
 
@@ -2180,6 +2251,516 @@ COMMENT ON COLUMN icinga_scheduled_downtime_range.range_type IS 'include -> rang
 COMMENT ON COLUMN icinga_scheduled_downtime_range.merge_behaviour IS 'set -> = {}, add -> += {}, substract -> -= {}';
 
 
+CREATE TABLE director_branch (
+  uuid  bytea NOT NULL UNIQUE CHECK(LENGTH(uuid) = 16),
+  owner character varying(255) NOT NULL,
+  branch_name character varying(255) NOT NULL,
+  description text DEFAULT NULL,
+  ts_merge_request bigint DEFAULT NULL,
+  PRIMARY KEY(uuid)
+);
+CREATE UNIQUE INDEX branch_branch_name ON director_branch (branch_name);
+
+CREATE TYPE enum_branch_action AS ENUM('create', 'modify', 'delete');
+
+CREATE TABLE director_branch_activity (
+  timestamp_ns bigint NOT NULL,
+  object_uuid bytea NOT NULL CHECK(LENGTH(object_uuid) = 16),
+  branch_uuid bytea NOT NULL CHECK(LENGTH(branch_uuid) = 16),
+  action enum_branch_action NOT NULL,
+  object_table character varying(64) NOT NULL,
+  author character varying(255) NOT NULL,
+  former_properties text NOT NULL,
+  modified_properties text NOT NULL,
+  PRIMARY KEY (timestamp_ns),
+  CONSTRAINT branch_activity_branch
+  FOREIGN KEY (branch_uuid)
+    REFERENCES director_branch (uuid)
+      ON DELETE CASCADE
+      ON UPDATE CASCADE
+);
+CREATE INDEX branch_activity_object_uuid ON director_branch_activity (object_uuid);
+CREATE INDEX branch_activity_branch_uuid ON director_branch_activity (branch_uuid);
+
+
+CREATE TABLE branched_icinga_host (
+  uuid bytea NOT NULL UNIQUE CHECK(LENGTH(uuid) = 16),
+  branch_uuid bytea NOT NULL CHECK(LENGTH(branch_uuid) = 16),
+  branch_created enum_boolean NOT NULL DEFAULT 'n',
+  branch_deleted enum_boolean NOT NULL DEFAULT 'n',
+
+  object_name character varying(255) DEFAULT NULL,
+  object_type enum_object_type_all DEFAULT NULL,
+  disabled enum_boolean DEFAULT NULL,
+  display_name CHARACTER VARYING(255) DEFAULT NULL,
+  address character varying(255) DEFAULT NULL,
+  address6 character varying(45) DEFAULT NULL,
+  check_command character varying(255) DEFAULT NULL,
+  max_check_attempts integer DEFAULT NULL,
+  check_period character varying(255) DEFAULT NULL,
+  check_interval character varying(8) DEFAULT NULL,
+  retry_interval character varying(8) DEFAULT NULL,
+  check_timeout smallint DEFAULT NULL,
+  enable_notifications enum_boolean DEFAULT NULL,
+  enable_active_checks enum_boolean DEFAULT NULL,
+  enable_passive_checks enum_boolean DEFAULT NULL,
+  enable_event_handler enum_boolean DEFAULT NULL,
+  enable_flapping enum_boolean DEFAULT NULL,
+  enable_perfdata enum_boolean DEFAULT NULL,
+  event_command character varying(255) DEFAULT NULL,
+  flapping_threshold_high smallint default null,
+  flapping_threshold_low smallint default null,
+  volatile enum_boolean DEFAULT NULL,
+  zone character varying(255) DEFAULT NULL,
+  command_endpoint character varying(255) DEFAULT NULL,
+  notes text DEFAULT NULL,
+  notes_url character varying(255) DEFAULT NULL,
+  action_url character varying(255) DEFAULT NULL,
+  icon_image character varying(255) DEFAULT NULL,
+  icon_image_alt character varying(255) DEFAULT NULL,
+  has_agent enum_boolean DEFAULT NULL,
+  master_should_connect enum_boolean DEFAULT NULL,
+  accept_config enum_boolean DEFAULT NULL,
+  custom_endpoint_name character varying(255) DEFAULT NULL,
+  api_key character varying(40) DEFAULT NULL,
+  -- template_choice character varying(255) DEFAULT NULL, -- TODO: Forbid them!
+
+  imports TEXT DEFAULT NULL,
+  groups TEXT DEFAULT NULL,
+  vars TEXT DEFAULT NULL,
+
+  set_null TEXT DEFAULT NULL,
+  PRIMARY KEY (branch_uuid, uuid),
+  CONSTRAINT icinga_host_branch
+    FOREIGN KEY (branch_uuid)
+    REFERENCES director_branch (uuid)
+    ON DELETE CASCADE
+    ON UPDATE CASCADE
+);
+
+CREATE UNIQUE INDEX host_branch_object_name ON branched_icinga_host (branch_uuid, object_name);
+CREATE INDEX branched_host_search_object_name ON branched_icinga_host (object_name);
+CREATE INDEX branched_host_search_display_name ON branched_icinga_host (display_name);
+
+
+CREATE TABLE branched_icinga_hostgroup (
+  uuid bytea NOT NULL UNIQUE CHECK(LENGTH(uuid) = 16),
+  branch_uuid bytea NOT NULL CHECK(LENGTH(branch_uuid) = 16),
+  branch_created enum_boolean NOT NULL DEFAULT 'n',
+  branch_deleted enum_boolean NOT NULL DEFAULT 'n',
+
+  object_name character varying(255) DEFAULT NULL,
+  object_type enum_object_type_all DEFAULT NULL,
+  disabled enum_boolean DEFAULT NULL,
+  display_name character varying(255) DEFAULT NULL,
+  assign_filter text DEFAULT NULL,
+  set_null TEXT DEFAULT NULL,
+  PRIMARY KEY (branch_uuid, uuid),
+  CONSTRAINT icinga_hostgroup_branch
+    FOREIGN KEY (branch_uuid)
+      REFERENCES director_branch (uuid)
+      ON DELETE CASCADE
+      ON UPDATE CASCADE
+);
+
+CREATE UNIQUE INDEX hostgroup_branch_object_name ON branched_icinga_hostgroup (branch_uuid, object_name);
+CREATE INDEX branched_hostgroup_search_object_name ON branched_icinga_hostgroup (object_name);
+CREATE INDEX branched_hostgroup_search_display_name ON branched_icinga_hostgroup (display_name);
+
+
+CREATE TABLE branched_icinga_servicegroup (
+  uuid bytea NOT NULL UNIQUE CHECK(LENGTH(uuid) = 16),
+  branch_uuid bytea NOT NULL CHECK(LENGTH(branch_uuid) = 16),
+  branch_created enum_boolean NOT NULL DEFAULT 'n',
+  branch_deleted enum_boolean NOT NULL DEFAULT 'n',
+
+  object_name character varying(255) DEFAULT NULL,
+  object_type enum_object_type_all DEFAULT NULL,
+  disabled enum_boolean DEFAULT NULL,
+  display_name character varying(255) DEFAULT NULL,
+  assign_filter text DEFAULT NULL,
+  set_null TEXT DEFAULT NULL,
+  PRIMARY KEY (branch_uuid, uuid),
+  CONSTRAINT icinga_servicegroup_branch
+    FOREIGN KEY (branch_uuid)
+      REFERENCES director_branch (uuid)
+      ON DELETE CASCADE
+      ON UPDATE CASCADE
+);
+
+CREATE UNIQUE INDEX servicegroup_branch_object_name ON branched_icinga_servicegroup (branch_uuid, object_name);
+CREATE INDEX branched_servicegroup_search_object_name ON branched_icinga_servicegroup (object_name);
+CREATE INDEX branched_servicegroup_search_display_name ON branched_icinga_servicegroup (display_name);
+
+
+CREATE TABLE branched_icinga_usergroup (
+  uuid bytea NOT NULL UNIQUE CHECK(LENGTH(uuid) = 16),
+  branch_uuid bytea NOT NULL CHECK(LENGTH(branch_uuid) = 16),
+  branch_created enum_boolean NOT NULL DEFAULT 'n',
+  branch_deleted enum_boolean NOT NULL DEFAULT 'n',
+
+  object_name character varying(255) DEFAULT NULL,
+  object_type enum_object_type_all DEFAULT NULL,
+  disabled enum_boolean DEFAULT NULL,
+  display_name character varying(255) DEFAULT NULL,
+  assign_filter text DEFAULT NULL,
+  set_null TEXT DEFAULT NULL,
+  PRIMARY KEY (branch_uuid, uuid),
+  CONSTRAINT icinga_usergroup_branch
+    FOREIGN KEY (branch_uuid)
+      REFERENCES director_branch (uuid)
+      ON DELETE CASCADE
+      ON UPDATE CASCADE
+);
+
+CREATE UNIQUE INDEX usergroup_branch_object_name ON branched_icinga_usergroup (branch_uuid, object_name);
+CREATE INDEX branched_usergroup_search_object_name ON branched_icinga_usergroup (object_name);
+CREATE INDEX branched_usergroup_search_display_name ON branched_icinga_usergroup (display_name);
+
+
+CREATE TABLE branched_icinga_user (
+  uuid bytea NOT NULL UNIQUE CHECK(LENGTH(uuid) = 16),
+  branch_uuid bytea NOT NULL CHECK(LENGTH(branch_uuid) = 16),
+  branch_created enum_boolean NOT NULL DEFAULT 'n',
+  branch_deleted enum_boolean NOT NULL DEFAULT 'n',
+
+  object_name character varying(255) DEFAULT NULL,
+  object_type enum_object_type_all DEFAULT NULL,
+  disabled enum_boolean DEFAULT NULL,
+  display_name character varying(255) DEFAULT NULL,
+  email character varying(255) DEFAULT NULL,
+  pager character varying(255) DEFAULT NULL,
+  enable_notifications enum_boolean DEFAULT NULL,
+  period character varying(255) DEFAULT NULL,
+  zone character varying(255) DEFAULT NULL,
+
+  imports TEXT DEFAULT NULL,
+  groups TEXT DEFAULT NULL,
+  vars TEXT DEFAULT NULL,
+  set_null TEXT DEFAULT NULL,
+  PRIMARY KEY (branch_uuid, uuid),
+  CONSTRAINT icinga_user_branch
+    FOREIGN KEY (branch_uuid)
+    REFERENCES director_branch (uuid)
+    ON DELETE CASCADE
+    ON UPDATE CASCADE
+);
+
+CREATE UNIQUE INDEX user_branch_object_name ON branched_icinga_user (branch_uuid, object_name);
+CREATE INDEX branched_user_search_object_name ON branched_icinga_user (object_name);
+CREATE INDEX branched_user_search_display_name ON branched_icinga_user (display_name);
+
+
+CREATE TABLE branched_icinga_zone (
+  uuid bytea NOT NULL UNIQUE CHECK(LENGTH(uuid) = 16),
+  branch_uuid bytea NOT NULL CHECK(LENGTH(branch_uuid) = 16),
+  branch_created enum_boolean NOT NULL DEFAULT 'n',
+  branch_deleted enum_boolean NOT NULL DEFAULT 'n',
+
+  object_name character varying(255) DEFAULT NULL,
+  parent character varying(255) DEFAULT NULL,
+  object_type enum_object_type_all DEFAULT NULL,
+  disabled enum_boolean DEFAULT NULL,
+  is_global enum_boolean DEFAULT NULL,
+
+  imports TEXT DEFAULT NULL,
+  set_null TEXT DEFAULT NULL,
+  PRIMARY KEY (branch_uuid, uuid),
+  CONSTRAINT icinga_zone_branch
+    FOREIGN KEY (branch_uuid)
+    REFERENCES director_branch (uuid)
+    ON DELETE CASCADE
+    ON UPDATE CASCADE
+);
+
+CREATE UNIQUE INDEX zone_branch_object_name ON branched_icinga_zone (branch_uuid, object_name);
+CREATE INDEX branched_zone_search_object_name ON branched_icinga_zone (object_name);
+
+
+CREATE TABLE branched_icinga_timeperiod (
+  uuid bytea NOT NULL UNIQUE CHECK(LENGTH(uuid) = 16),
+  branch_uuid bytea NOT NULL CHECK(LENGTH(branch_uuid) = 16),
+  branch_created enum_boolean NOT NULL DEFAULT 'n',
+  branch_deleted enum_boolean NOT NULL DEFAULT 'n',
+
+  object_name character varying(255) DEFAULT NULL,
+  display_name character varying(255) DEFAULT NULL,
+  update_method character varying(64) DEFAULT NULL,
+  zone character varying(255) DEFAULT NULL,
+  object_type enum_object_type_all DEFAULT NULL,
+  disabled enum_boolean DEFAULT NULL,
+  prefer_includes enum_boolean DEFAULT NULL,
+
+  imports TEXT DEFAULT NULL,
+  ranges TEXT DEFAULT NULL,
+  set_null TEXT DEFAULT NULL,
+  PRIMARY KEY (branch_uuid, uuid),
+  CONSTRAINT icinga_timeperiod_branch
+    FOREIGN KEY (branch_uuid)
+      REFERENCES director_branch (uuid)
+      ON DELETE CASCADE
+      ON UPDATE CASCADE
+);
+
+CREATE UNIQUE INDEX timeperiod_branch_object_name ON branched_icinga_timeperiod (branch_uuid, object_name);
+CREATE INDEX branched_timeperiod_search_object_name ON branched_icinga_timeperiod (object_name);
+CREATE INDEX branched_timeperiod_search_display_name ON branched_icinga_timeperiod (display_name);
+
+
+CREATE TABLE branched_icinga_command (
+  uuid bytea NOT NULL UNIQUE CHECK(LENGTH(uuid) = 16),
+  branch_uuid bytea NOT NULL CHECK(LENGTH(branch_uuid) = 16),
+  branch_created enum_boolean NOT NULL DEFAULT 'n',
+  branch_deleted enum_boolean NOT NULL DEFAULT 'n',
+
+  object_name character varying(255) DEFAULT NULL,
+  object_type enum_object_type_all DEFAULT NULL,
+  disabled enum_boolean NOT NULL DEFAULT NULL,
+  methods_execute character varying(64) DEFAULT NULL,
+  command text DEFAULT NULL,
+  is_string enum_boolean DEFAULT NULL,
+-- env text DEFAULT NULL,
+  timeout smallint DEFAULT NULL,
+  zone character varying(255) DEFAULT NULL,
+
+  imports TEXT DEFAULT NULL,
+  arguments TEXT DEFAULT NULL,
+  set_null TEXT DEFAULT NULL,
+  PRIMARY KEY (branch_uuid, uuid),
+  CONSTRAINT icinga_command_branch
+      FOREIGN KEY (branch_uuid)
+          REFERENCES director_branch (uuid)
+          ON DELETE CASCADE
+          ON UPDATE CASCADE
+);
+
+CREATE UNIQUE INDEX command_branch_object_name ON branched_icinga_command (branch_uuid, object_name);
+CREATE INDEX branched_command_search_object_name ON branched_icinga_command (object_name);
+
+
+CREATE TABLE branched_icinga_apiuser (
+  uuid bytea NOT NULL UNIQUE CHECK(LENGTH(uuid) = 16),
+  branch_uuid bytea NOT NULL CHECK(LENGTH(branch_uuid) = 16),
+  branch_created enum_boolean NOT NULL DEFAULT 'n',
+  branch_deleted enum_boolean NOT NULL DEFAULT 'n',
+
+  object_name CHARACTER VARYING(255) DEFAULT NULL,
+  object_type enum_object_type_all DEFAULT NULL,
+  disabled enum_boolean NOT NULL DEFAULT NULL,
+  password CHARACTER VARYING(255) DEFAULT NULL,
+  client_dn CHARACTER VARYING(64) DEFAULT NULL,
+  permissions TEXT DEFAULT NULL,
+
+  set_null TEXT DEFAULT NULL,
+  PRIMARY KEY (branch_uuid, uuid),
+  CONSTRAINT icinga_apiuser_branch
+      FOREIGN KEY (branch_uuid)
+          REFERENCES director_branch (uuid)
+          ON DELETE CASCADE
+          ON UPDATE CASCADE
+);
+
+CREATE UNIQUE INDEX apiuser_branch_object_name ON branched_icinga_apiuser (branch_uuid, object_name);
+CREATE INDEX branched_apiuser_search_object_name ON branched_icinga_apiuser (object_name);
+
+
+CREATE TABLE branched_icinga_endpoint (
+  uuid bytea NOT NULL UNIQUE CHECK(LENGTH(uuid) = 16),
+  branch_uuid bytea NOT NULL CHECK(LENGTH(branch_uuid) = 16),
+  branch_created enum_boolean NOT NULL DEFAULT 'n',
+  branch_deleted enum_boolean NOT NULL DEFAULT 'n',
+
+  zone character varying(255) DEFAULT NULL,
+  object_name character varying(255) DEFAULT NULL,
+  object_type enum_object_type_all DEFAULT NULL,
+  disabled enum_boolean NOT NULL DEFAULT NULL,
+  host character varying(255) DEFAULT NULL,
+  port d_smallint DEFAULT NULL,
+  log_duration character varying(32) DEFAULT NULL,
+  apiuser character varying(255) DEFAULT NULL,
+
+  imports TEXT DEFAULT NULL,
+  set_null TEXT DEFAULT NULL,
+  PRIMARY KEY (branch_uuid, uuid),
+  CONSTRAINT icinga_endpoint_branch
+      FOREIGN KEY (branch_uuid)
+          REFERENCES director_branch (uuid)
+          ON DELETE CASCADE
+          ON UPDATE CASCADE
+);
+
+CREATE UNIQUE INDEX endpoint_branch_object_name ON branched_icinga_endpoint (branch_uuid, object_name);
+CREATE INDEX branched_endpoint_search_object_name ON branched_icinga_endpoint (object_name);
+
+
+CREATE TABLE branched_icinga_service (
+  uuid bytea NOT NULL UNIQUE CHECK(LENGTH(uuid) = 16),
+  branch_uuid bytea NOT NULL CHECK(LENGTH(branch_uuid) = 16),
+  branch_created enum_boolean NOT NULL DEFAULT 'n',
+  branch_deleted enum_boolean NOT NULL DEFAULT 'n',
+
+  object_name character varying(255) DEFAULT NULL,
+  object_type enum_object_type_all DEFAULT NULL,
+  disabled enum_boolean DEFAULT NULL,
+  display_name character varying(255) DEFAULT NULL,
+  host character varying(255) DEFAULT NULL,
+  service_set character varying(255) DEFAULT NULL,
+  check_command character varying(255) DEFAULT NULL,
+  max_check_attempts integer DEFAULT NULL,
+  check_period character varying(255) DEFAULT NULL,
+  check_interval character varying(8) DEFAULT NULL,
+  retry_interval character varying(8) DEFAULT NULL,
+  check_timeout smallint DEFAULT NULL,
+  enable_notifications enum_boolean DEFAULT NULL,
+  enable_active_checks enum_boolean DEFAULT NULL,
+  enable_passive_checks enum_boolean DEFAULT NULL,
+  enable_event_handler enum_boolean DEFAULT NULL,
+  enable_flapping enum_boolean DEFAULT NULL,
+  enable_perfdata enum_boolean DEFAULT NULL,
+  event_command character varying(255) DEFAULT NULL,
+  flapping_threshold_high smallint DEFAULT NULL,
+  flapping_threshold_low smallint DEFAULT NULL,
+  volatile enum_boolean DEFAULT NULL,
+  zone character varying(255) DEFAULT NULL,
+  command_endpoint character varying(255) DEFAULT NULL,
+  notes text DEFAULT NULL,
+  notes_url character varying(255) DEFAULT NULL,
+  action_url character varying(255) DEFAULT NULL,
+  icon_image character varying(255) DEFAULT NULL,
+  icon_image_alt character varying(255) DEFAULT NULL,
+  use_agent enum_boolean DEFAULT NULL,
+  apply_for character varying(255) DEFAULT NULL,
+  use_var_overrides enum_boolean DEFAULT NULL,
+  assign_filter text DEFAULT NULL,
+  -- template_choice_id int DEFAULT NULL,
+
+  imports TEXT DEFAULT NULL,
+  groups TEXT DEFAULT NULL,
+  vars TEXT DEFAULT NULL,
+  set_null TEXT DEFAULT NULL,
+  PRIMARY KEY (branch_uuid, uuid),
+  CONSTRAINT icinga_service_branch
+      FOREIGN KEY (branch_uuid)
+          REFERENCES director_branch (uuid)
+          ON DELETE CASCADE
+          ON UPDATE CASCADE
+);
+
+CREATE UNIQUE INDEX service_branch_object_name ON branched_icinga_service (branch_uuid, object_name);
+CREATE INDEX branched_service_search_object_name ON branched_icinga_service (object_name);
+CREATE INDEX branched_service_search_display_name ON branched_icinga_service (display_name);
+
+
+CREATE TABLE branched_icinga_notification (
+  uuid bytea NOT NULL UNIQUE CHECK(LENGTH(uuid) = 16),
+  branch_uuid bytea NOT NULL CHECK(LENGTH(branch_uuid) = 16),
+  branch_created enum_boolean NOT NULL DEFAULT 'n',
+  branch_deleted enum_boolean NOT NULL DEFAULT 'n',
+
+  object_name CHARACTER VARYING(255) DEFAULT NULL,
+  object_type enum_object_type_all DEFAULT NULL,
+  disabled enum_boolean DEFAULT NULL,
+  apply_to enum_host_service DEFAULT NULL,
+  host character varying(255) DEFAULT NULL,
+  service character varying(255) DEFAULT NULL,
+  times_begin integer DEFAULT NULL,
+  times_end integer DEFAULT NULL,
+  notification_interval integer DEFAULT NULL,
+  command character varying(255) DEFAULT NULL,
+  period character varying(255) DEFAULT NULL,
+  zone character varying(255) DEFAULT NULL,
+  assign_filter text DEFAULT NULL,
+
+  states TEXT DEFAULT NULL,
+  types TEXT DEFAULT NULL,
+  users TEXT DEFAULT NULL,
+  usergroups TEXT DEFAULT NULL,
+
+  imports TEXT DEFAULT NULL,
+  vars TEXT DEFAULT NULL,
+  set_null TEXT DEFAULT NULL,
+  PRIMARY KEY (branch_uuid, uuid),
+  CONSTRAINT icinga_notification_branch
+      FOREIGN KEY (branch_uuid)
+          REFERENCES director_branch (uuid)
+          ON DELETE CASCADE
+          ON UPDATE CASCADE
+);
+
+CREATE UNIQUE INDEX notification_branch_object_name ON branched_icinga_notification (branch_uuid, object_name);
+CREATE INDEX branched_notification_search_object_name ON branched_icinga_notification (object_name);
+
+
+CREATE TABLE branched_icinga_scheduled_downtime (
+  uuid bytea NOT NULL UNIQUE CHECK(LENGTH(uuid) = 16),
+  branch_uuid bytea NOT NULL CHECK(LENGTH(branch_uuid) = 16),
+  branch_created enum_boolean NOT NULL DEFAULT 'n',
+  branch_deleted enum_boolean NOT NULL DEFAULT 'n',
+
+  object_name character varying(255) DEFAULT NULL,
+  zone_id integer DEFAULT NULL,
+  object_type enum_object_type_all DEFAULT NULL,
+  disabled enum_boolean DEFAULT NULL,
+  apply_to enum_host_service DEFAULT NULL,
+  assign_filter text DEFAULT NULL,
+  author character varying(255) DEFAULT NULL,
+  comment text DEFAULT NULL,
+  fixed enum_boolean DEFAULT NULL,
+  duration int DEFAULT NULL,
+  with_services enum_boolean DEFAULT NULL,
+
+  imports TEXT DEFAULT NULL,
+  ranges TEXT DEFAULT NULL,
+  set_null TEXT DEFAULT NULL,
+  PRIMARY KEY (branch_uuid, uuid),
+  CONSTRAINT icinga_scheduled_downtime_branch
+      FOREIGN KEY (branch_uuid)
+          REFERENCES director_branch (uuid)
+          ON DELETE CASCADE
+          ON UPDATE CASCADE
+);
+
+CREATE UNIQUE INDEX scheduled_downtime_branch_object_name ON branched_icinga_scheduled_downtime (branch_uuid, object_name);
+CREATE INDEX branched_scheduled_downtime_search_object_name ON branched_icinga_scheduled_downtime (object_name);
+
+
+CREATE TABLE branched_icinga_dependency (
+  uuid bytea NOT NULL UNIQUE CHECK(LENGTH(uuid) = 16),
+  branch_uuid bytea NOT NULL CHECK(LENGTH(branch_uuid) = 16),
+  branch_created enum_boolean NOT NULL DEFAULT 'n',
+  branch_deleted enum_boolean NOT NULL DEFAULT 'n',
+
+  object_name character varying(255) NOT NULL,
+  object_type enum_object_type_all NOT NULL,
+  disabled enum_boolean DEFAULT 'n',
+  apply_to enum_host_service NULL DEFAULT NULL,
+  parent_host character varying(255) DEFAULT NULL,
+  parent_host_var character varying(128) DEFAULT NULL,
+  parent_service character varying(255) DEFAULT NULL,
+  child_host character varying(255) DEFAULT NULL,
+  child_service character varying(255) DEFAULT NULL,
+  disable_checks enum_boolean DEFAULT NULL,
+  disable_notifications enum_boolean DEFAULT NULL,
+  ignore_soft_states enum_boolean DEFAULT NULL,
+  period_id integer DEFAULT NULL,
+  zone_id integer DEFAULT NULL,
+  assign_filter text DEFAULT NULL,
+  parent_service_by_name character varying(255),
+
+  imports TEXT DEFAULT NULL,
+  set_null TEXT DEFAULT NULL,
+  PRIMARY KEY (branch_uuid, uuid),
+  CONSTRAINT icinga_dependency_branch
+      FOREIGN KEY (branch_uuid)
+          REFERENCES director_branch (uuid)
+          ON DELETE CASCADE
+          ON UPDATE CASCADE
+);
+
+CREATE UNIQUE INDEX dependency_branch_object_name ON branched_icinga_dependency (branch_uuid, object_name);
+CREATE INDEX branched_dependency_search_object_name ON branched_icinga_dependency (object_name);
+
+
 INSERT INTO director_schema_migration
   (schema_version, migration_time)
-  VALUES (167, NOW());
+  VALUES (178, NOW());
